unit AddCtrls;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, Menus, ExtCtrls, DsgnIntf, EditMan, Mask;

type
  TAddControlsDialog = class(TForm)
    AddToGroup: TGroupBox;
    MenuCombo: TComboBox;
    PopupCombo: TComboBox;
    PanelCombo: TComboBox;
    MenuLabel: TLabel;
    PopupLabel: TLabel;
    PanelLabel: TLabel;
    CancelBtn: TButton;
    OKBtn: TButton;
    MenuGroup: TGroupBox;
    ButtonGroup: TGroupBox;
    ShortCutCheck: TCheckBox;
    FuncKeysRadio: TRadioButton;
    AccelKeysRadio: TRadioButton;
    LeftLabel: TLabel;
    TopLabel: TLabel;
    HintCheck: TCheckBox;
    ColorCheck: TCheckBox;
    LeftEdit: TEdit;
    TopEdit: TEdit;
    HeightLabel: TLabel;
    WidthLabel: TLabel;
    HeightEdit: TEdit;
    WidthEdit: TEdit;
    procedure FormActivate(Sender: TObject);
    procedure MenuComboChange(Sender: TObject);
    procedure ComboKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure OKBtnClick(Sender: TObject);
    procedure PanelComboChange(Sender: TObject);
    procedure ShortCutCheckClick(Sender: TObject);
    procedure EditExit(Sender: TObject);
    procedure EditClick(Sender: TObject);
  private
    procedure AddMenuCtrls(Form: TForm; Menu: TMainMenu);
    procedure AddPopupCtrls(Form: TForm; Popup: TPopupMenu);
    procedure AddToolbarCtrls(Form: TForm; Panel: TPanel);
    procedure EnableControls(GroupBox: TGroupBox);
  public
    EditManager: TEditManager;
    Designer: TFormDesigner;
  end;

  TAddControlsEditor = class(TComponentEditor)
  public
    procedure Edit; override;
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

var
  AddControlsDialog: TAddControlsDialog;

implementation

{$R *.DFM}

const
  EditNames: array[erInsert..erCancel] of string =
    ('Insert', 'Delete', 'Edit', 'Post', 'Cancel');

procedure TAddControlsDialog.FormActivate(Sender: TObject);
var
  i: Integer;
begin
  with EditManager.Owner do
    for i := 0 to ComponentCount-1 do
      if (Components[i] is TMainMenu) then
        MenuCombo.Items.Add(Components[i].Name)
      else if (Components[i] is TPopupMenu) then
        PopupCombo.Items.Add(Components[i].Name)
      else if (Components[i] is TPanel) then
        PanelCombo.Items.Add(Components[i].Name);
end;

procedure TAddControlsDialog.EnableControls(GroupBox: TGroupBox);
var
  i: Integer;
begin
  with GroupBox do begin
    if Enabled then
      Font.Color := clBlack else
      Font.Color := clGray;
    for i := 0 to ControlCount-1 do
      Controls[i].Enabled := Enabled;
  end;
end;

procedure TAddControlsDialog.MenuComboChange(Sender: TObject);
begin
  MenuGroup.Enabled := MenuCombo.ItemIndex > -1;
  EnableControls(MenuGroup);
  ShortCutCheckClick(nil);
end;

procedure TAddControlsDialog.PanelComboChange(Sender: TObject);
begin
  ButtonGroup.Enabled := PanelCombo.ItemIndex > -1;
  EnableControls(ButtonGroup);
end;

procedure TAddControlsDialog.ComboKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_DELETE then
    with Sender as TComboBox do begin
      ItemIndex := -1;
      if Assigned(OnChange) then OnChange(Sender);
    end;
end;

procedure TAddControlsDialog.ShortCutCheckClick(Sender: TObject);
begin
  FuncKeysRadio.Enabled := MenuGroup.Enabled and ShortCutCheck.Checked;
  AccelKeysRadio.Enabled := FuncKeysRadio.Enabled;
end;

procedure TAddControlsDialog.EditExit(Sender: TObject);
begin
  with Sender as TEdit do
    if StrToIntDef(Text, -1) < 0 then    {then illegal}
      Text := IntToStr(Tag);             {so restore default}
end;

procedure TAddControlsDialog.EditClick(Sender: TObject);
begin
  EditExit(Sender);
end;

procedure TAddControlsDialog.OKBtnClick(Sender: TObject);
var
  f: TForm;
  mm: TMainMenu;
  pm: TPopupMenu;
  tp: TPanel;
begin
  f := EditManager.Owner as TForm;
  with MenuCombo do
    if ItemIndex > -1 then begin
      mm := (f.FindComponent(Items[ItemIndex]) as TMainMenu);
      if mm <> nil then AddMenuCtrls(f, mm);
    end;
  with PopupCombo do
    if ItemIndex > -1 then begin
      pm := (f.FindComponent(Items[ItemIndex]) as TPopupMenu);
      if pm <> nil then AddPopupCtrls(f, pm);
    end;
  with PanelCombo do
    if ItemIndex > -1 then begin
      tp := (f.FindComponent(Items[ItemIndex]) as TPanel);
      if tp <> nil then AddToolbarCtrls(f, tp);
    end;
end;

procedure TAddControlsDialog.AddMenuCtrls(Form: TForm; Menu: TMainMenu);
  procedure AddItem(Index: Integer; EditMenu: TMenuItem);
  var
    mi: TMenuItem;
  begin
    mi := Designer.CreateComponent(TMenuItem, EditMenu, 0, 0, 0, 0) as TMenuItem;
    mi.Caption := '&' + EditNames[Index];
    if ShortCutCheck.Checked then
      if FuncKeysRadio.Checked then
        mi.ShortCut := Menus.ShortCut(Word(VK_F2 + Index), []) else
        mi.ShortCut := Menus.ShortCut(Word(EditNames[Index][1]), [ssCtrl]);
    EditMenu.Add(mi);
    EditManager.MenuItems.Refs[Index] := mi;
  end;
var
  em: TMenuItem;
begin
  {first add menu bar Edit item...}
  em := Designer.CreateComponent(TMenuItem, Menu, 0, 0, 0, 0) as TMenuItem;
  em.Caption := '&' + EditNames[erEdit];
  Menu.Items.Insert(1, em);                  {insert in second position}
  AddItem(erInsert, em);
  AddItem(erDelete, em);
  AddItem(erEdit,   em);
  AddItem(erPost,   em);
  AddItem(erCancel, em);
end;

procedure TAddControlsDialog.AddPopupCtrls(Form: TForm; Popup: TPopupMenu);
  procedure AddItem(Index: Integer);
  var
    mi: TMenuItem;
  begin
    mi := Designer.CreateComponent(TMenuItem, Popup, 0, 0, 0, 0) as TMenuItem;
    mi.Caption := EditNames[Index];
    Popup.Items.Add(mi);
    EditManager.PopupItems.Refs[Index] := mi;
  end;
begin
  AddItem(erInsert);
  AddItem(erDelete);
  AddItem(erEdit);
  AddItem(erPost);
  AddItem(erCancel);
end;

procedure TAddControlsDialog.AddToolbarCtrls(Form: TForm; Panel: TPanel);
var
  LeftOffset, TopOffset, BtnHeight, BtnWidth: Integer;
  ColorChar: Char;
  procedure AddBtn(Index: Integer);
  var
    sb: TSpeedButton;
  begin
    sb := Designer.CreateComponent(
      TSpeedButton, Panel, 0, 0, BtnWidth, BtnHeight) as TSpeedButton;
    with sb do begin
      {reallign afterwards as designer rounds off to nearest grid boundary}
      if Panel.Width >= Panel.Height then begin   {then align horzontally...}
        Left := (Index * BtnWidth) + LeftOffset;
        Top  := TopOffset;
      end
      else begin                                 {else align vertically...}
        Left := LeftOffset;
        Top  := (Index * BtnHeight) + TopOffset;
      end;
      NumGlyphs := 2;
      Glyph.Handle := LoadBitmap(HInstance,
        PChar('EB' + ColorChar + '_' + UpperCase(EditNames[Index])));
      if HintCheck.Checked then begin
        Hint := EditNames[Index] + ' record';
        ShowHint := True;
      end;
    end;
    EditManager.SpeedButtons.Refs[Index] := sb;
  end;
begin
{ if Designer = nil then
    raise EComponentError.Create('No Designer');}
  LeftOffset := StrToIntDef(LeftEdit.Text, 4);
  TopOffset  := StrToIntDef(TopEdit.Text, 4);
  BtnHeight  := StrToIntDef(HeightEdit.Text, 25);
  BtnWidth   := StrToIntDef(WidthEdit.Text, 25);
  if ColorCheck.Checked then
    ColorChar := 'C' else                 {colour}
    ColorChar := 'M';                     {monochrome}
  AddBtn(erInsert);
  AddBtn(erDelete);
  AddBtn(erEdit);
  AddBtn(erPost);
  AddBtn(erCancel);
end;

{TAddControlsEditor}

procedure TAddControlsEditor.Edit;
begin
  with TAddControlsDialog.Create(Application) do
  try
    EditManager := Component as TEditManager;
    Designer := Self.Designer;
    ShowModal;
  finally
    Free;
  end;
end;

procedure TAddControlsEditor.ExecuteVerb(Index: Integer);
begin
  if Index = 0 then Edit;
end;

function TAddControlsEditor.GetVerb(Index: Integer): string;
begin
  if Index = 0 then Result := '&Add Edit Controls...';
end;

function TAddControlsEditor.GetVerbCount: Integer;
begin
  Result := 1;
end;

end.
